@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -50,6 +50,7 @@
 
 /* -- path_list_async_new implementation -- */
 
+
 PathListData * 
 path_list_data_new ()
 {
@@ -116,6 +117,8 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 	pli->result = result;
 
 	if (pli->interrupted) {
+		gnome_vfs_async_cancel (handle);
+		pli->interrupted = FALSE;
 		if (pli->interrupt_func) 
 			pli->interrupt_func (pli->interrupt_data);
 		path_list_data_free (pli);
@@ -123,17 +126,16 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 	}
 
 	for (node = list; node != NULL; node = node->next) {
-		GnomeVFSFileInfo * info     = node->data;
-		GnomeVFSURI *      full_uri = NULL;
-		gchar *            str_uri;
-		gchar *            unesc_uri;
+		GnomeVFSFileInfo *info     = node->data;
+		GnomeVFSURI      *full_uri = NULL;
+		char             *str_uri;
+		char             *unesc_uri;
 
 		switch (info->type) {
 		case GNOME_VFS_FILE_TYPE_REGULAR:
 			full_uri = gnome_vfs_uri_append_file_name (pli->uri, info->name);
 			str_uri = gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
 			unesc_uri = gnome_vfs_unescape_string (str_uri, NULL);
-
 			pli->files = g_list_prepend (pli->files, unesc_uri);
 			g_free (str_uri);
 			break;
@@ -172,18 +174,31 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 
 
 PathListHandle *
-path_list_async_new (const gchar      *uri, 
+path_list_async_new (const char       *uri, 
 		     PathListDoneFunc  f,
 		     gpointer         data)
 {
 	GnomeVFSAsyncHandle *handle;
-	PathListData *pli;
-	gchar *escaped;
-	PathListHandle *pl_handle;
+	PathListData        *pli;
+	char                *escaped;
+	PathListHandle      *pl_handle;
+
+	if (uri == NULL) {
+		if (f != NULL)
+			(f) (NULL, data);
+		return NULL;
+	}
 
 	pli = path_list_data_new ();
 
 	escaped = gnome_vfs_escape_path_string (uri);
+	if (escaped == NULL) {
+		path_list_data_free (pli);
+		if (f != NULL)
+			(f) (NULL, data);
+		return NULL;
+	}
+
 	pli->uri = gnome_vfs_uri_new (escaped);
 	g_free (escaped);
 
@@ -298,15 +313,16 @@ typedef struct {
 	gpointer data;
 	GList *dirs;
 	GList *visited_dirs;
+	gboolean interrupted;
 } VisitRCDirData;
 
 
-VisitRCDirData *
+static VisitRCDirData *
 visit_rc_dir_data_new ()
 {
 	VisitRCDirData * rcd;
 
-	rcd = g_new (VisitRCDirData, 1);
+	rcd = g_new0 (VisitRCDirData, 1);
 	rcd->dirs = NULL;
 	rcd->visited_dirs = NULL;
 
@@ -314,7 +330,7 @@ visit_rc_dir_data_new ()
 }
 
 
-void
+static void
 visit_rc_dir_data_free (VisitRCDirData *rcd)
 {
 	if (rcd == NULL)
@@ -372,9 +388,12 @@ rc_path_list_done_cb (PathListData *pld,
 			(* rcd->do_something) (real_file, rc_file, rcd->data);
 
 		g_free (real_file);
+
+		if (rcd->interrupted)
+			break;
 	}
 
-	if (! rcd->recursive) {
+	if (! rcd->recursive || rcd->interrupted) {
 		if (rcd->done_func)
 			(* rcd->done_func) (rcd->visited_dirs, rcd->data);
 		path_list_data_free (pld);
@@ -874,7 +893,7 @@ image_is_jpeg (const char *name)
 {
 	const char *result;
 
-	if (eel_gconf_get_boolean (PREF_FAST_FILE_TYPE))
+	if (eel_gconf_get_boolean (PREF_FAST_FILE_TYPE, TRUE))
 		result = gnome_vfs_mime_type_from_name_or_default (name, NULL);
 	else 
 		result = gnome_vfs_get_file_mime_type (name, NULL, FALSE);
