@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,14 +39,23 @@
 
 #include "typedefs.h"
 #include "main.h"
+#include "glib-utils.h"
 #include "gthumb-window.h"
 #include "gtk-utils.h"
-#include "image-list.h"
-#include "image-list-utils.h"
+#include "gth-file-view.h"
 #include "comments.h"
 
 
-#define COMMENT_GLADE_FILE "gthumb_comments.glade"
+enum {
+	NO_DATE = 0,
+	FOLLOWING_DATE,
+	CURRENT_DATE,
+	IMAGE_CREATION_DATE,
+	EXIF_DATE
+};
+
+
+#define GLADE_FILE "gthumb_comments.glade"
 
 typedef struct {
 	GThumbWindow  *window;
@@ -54,28 +63,30 @@ typedef struct {
 	GList         *file_list;
 
 	GtkWidget     *dialog;
-	GtkWidget     *place_entry;
-	GtkWidget     *date_checkbutton;
-	GtkWidget     *date_dateedit;
-	GtkWidget     *date_today_button;
-	GtkWidget     *date_ctime_button;
-	GtkWidget     *date_exif_button;
 
 	GtkWidget     *note_text_view;
 	GtkTextBuffer *note_text_buffer;
 
+	GtkWidget     *place_entry;
+
+	GtkWidget     *date_optionmenu;
+	GtkWidget     *date_dateedit;
+
 	GtkWidget     *save_changed_checkbutton;
 
 	CommentData   *original_cdata;
 	gboolean       have_exif_data;
+	gboolean       several_images;
 } DialogData;
 
 
 /* called when the main dialog is closed. */
 static void
-destroy_cb (GtkWidget *widget, 
+destroy_cb (GtkWidget  *widget, 
 	    DialogData *data)
 {
+	data->window->comments_dlg = NULL;
+
 	g_object_unref (data->gui);
 	g_list_foreach (data->file_list, (GFunc) g_free, NULL);
 	g_list_free (data->file_list);
@@ -85,21 +96,6 @@ destroy_cb (GtkWidget *widget,
 }
 
 
-static void
-date_check_toggled_cb (GtkWidget *widget,
-		       DialogData *data)
-{
-	gtk_widget_set_sensitive (data->date_dateedit,
-				  GTK_TOGGLE_BUTTON (widget)->active);
-	gtk_widget_set_sensitive (data->date_today_button,
-				  GTK_TOGGLE_BUTTON (widget)->active);
-	gtk_widget_set_sensitive (data->date_ctime_button,
-				  GTK_TOGGLE_BUTTON (widget)->active);
-	gtk_widget_set_sensitive (data->date_exif_button,
-				  GTK_TOGGLE_BUTTON (widget)->active && data->have_exif_data);
-}
-
-
 static int
 strcmp_null_tollerant (const char *s1, const char *s2)
 {
@@ -126,6 +122,94 @@ text_field_cmp (const char *s1, const char *s2)
 }
 
 
+#ifdef HAVE_LIBEXIF
+
+static time_t
+get_exif_date (DialogData *data, 
+	       const char *filename)
+{
+	ExifData     *edata;
+	unsigned int  i, j;
+	time_t        time = 0;
+	struct tm     tm = { 0, };
+
+	edata = exif_data_new_from_file (filename);
+	if (edata == NULL) 
+		return (time_t) 0;
+
+	for (i = 0; i < EXIF_IFD_COUNT; i++) {
+		ExifContent *content = edata->ifd[i];
+
+		if (! edata->ifd[i] || ! edata->ifd[i]->count) 
+			continue;
+
+		for (j = 0; j < content->count; j++) {
+			ExifEntry *e = content->entries[j];
+			char      *data;
+
+			if (! content->entries[j]) 
+				continue;
+
+			if ((e->tag != EXIF_TAG_DATE_TIME) &&
+			    (e->tag != EXIF_TAG_DATE_TIME_ORIGINAL) &&
+			    (e->tag != EXIF_TAG_DATE_TIME_DIGITIZED))
+				continue;
+
+			data = g_strdup (e->data);
+			data[4] = data[7] = data[10] = data[13] = data[16] = '\0';
+
+			tm.tm_year = atoi (data) - 1900;
+			tm.tm_mon  = atoi (data + 5) - 1;
+			tm.tm_mday = atoi (data + 8);
+			tm.tm_hour = atoi (data + 11);
+			tm.tm_min  = atoi (data + 14);
+			tm.tm_sec  = atoi (data + 17);
+			time = mktime (&tm);
+
+			g_free (data);
+
+			break;
+		}
+	}
+
+	exif_data_unref (edata);
+
+	return time;
+}
+
+#endif
+
+
+static time_t
+get_requested_time (DialogData *data, 
+		    const char *filename)
+{
+	int idx  = gtk_option_menu_get_history (GTK_OPTION_MENU (data->date_optionmenu));
+	time_t t = (time_t)0;
+
+	switch (idx) {
+	case NO_DATE:
+		break;
+	case FOLLOWING_DATE:
+		t = gnome_date_edit_get_time (GNOME_DATE_EDIT (data->date_dateedit));
+		break;
+	case CURRENT_DATE:
+		t = time (NULL);
+		break;
+	case IMAGE_CREATION_DATE:
+		t = get_file_ctime (filename);
+		break;
+	case EXIF_DATE:
+#ifdef HAVE_LIBEXIF
+		t = get_exif_date (data, filename);
+#endif
+		break;
+	}
+
+	return t;
+}
+
+
 /* called when the "ok" button is pressed. */
 static void
 ok_clicked_cb (GtkWidget  *widget, 
@@ -146,28 +230,8 @@ ok_clicked_cb (GtkWidget  *widget,
 
 	text = gtk_entry_get_text (GTK_ENTRY (data->place_entry));
 	if (text != NULL)
-		cdata->place = g_locale_from_utf8 (text, -1, NULL, NULL, NULL);
+		cdata->place = g_strdup (text);
 	
-	/* Date */
-
-	if (GTK_TOGGLE_BUTTON (data->date_checkbutton)->active) {
-		time_t     t;
-		struct tm *tm_date;
-
-		t = gnome_date_edit_get_time (GNOME_DATE_EDIT (data->date_dateedit));
-		tm_date = localtime (&t);
-		tm.tm_year = tm_date->tm_year;
-		tm.tm_mon = tm_date->tm_mon;
-		tm.tm_mday = tm_date->tm_mday;
-		tm.tm_hour = tm_date->tm_hour;
-		tm.tm_min = tm_date->tm_min;
-		tm.tm_sec = tm_date->tm_sec;
-		tm.tm_isdst = tm_date->tm_isdst;
-	}
-	cdata->time = mktime (&tm);
-	if (cdata->time <= 0)
-		cdata->time = 0;
-
 	/* Comment */
 
 	gtk_text_buffer_get_bounds (data->note_text_buffer, 
@@ -177,10 +241,8 @@ ok_clicked_cb (GtkWidget  *widget,
 					 &start_iter, 
 					 &end_iter, 
 					 FALSE);
-
-	if (comment_text != NULL)
-		cdata->comment = g_locale_from_utf8 (comment_text, -1, NULL, NULL, NULL);
-	g_free (comment_text);
+	if (comment_text)
+		cdata->comment = comment_text;
 
 	/**/
 
@@ -209,12 +271,38 @@ ok_clicked_cb (GtkWidget  *widget,
 
 	for (scan = data->file_list; scan; scan = scan->next) {
 		const char *filename = scan->data;
+
+		/* Date */
+
+		cdata->time = -1;
+
+		if (gtk_option_menu_get_history (GTK_OPTION_MENU (data->date_optionmenu)) != NO_DATE) {
+			time_t     t;
+			struct tm *tm_date;
+			
+			t = get_requested_time (data, filename);
+			tm_date = localtime (&t);
+			tm.tm_year = tm_date->tm_year;
+			tm.tm_mon = tm_date->tm_mon;
+			tm.tm_mday = tm_date->tm_mday;
+			tm.tm_hour = tm_date->tm_hour;
+			tm.tm_min = tm_date->tm_min;
+			tm.tm_sec = tm_date->tm_sec;
+			tm.tm_isdst = tm_date->tm_isdst;
+		}
+		cdata->time = mktime (&tm);
+		if (cdata->time <= 0)
+			cdata->time = 0;
+
+		/**/
+
 		if (save_changed)
 			comments_save_comment_non_null (filename, cdata);
 		else
 			comments_save_comment (filename, cdata);
 		all_windows_notify_update_comment (filename); 
 	}
+
 	comment_data_free (cdata);
 	gtk_widget_destroy (data->dialog);
 }
@@ -254,85 +342,56 @@ help_cb (GtkWidget  *widget,
 
 
 static void
-date_today_clicked_cb (GtkWidget  *widget, 
-		       DialogData *data)
-{
-	gnome_date_edit_set_time (GNOME_DATE_EDIT (data->date_dateedit),
-				  time (NULL));
-}
-
-
-static void
-date_ctime_clicked_cb (GtkWidget  *widget, 
-		       DialogData *data)
+date_optionmenu_changed_cb (GtkOptionMenu *option_menu,
+			    DialogData    *data)
 {
 	char *first_image = data->file_list->data;
+	int   idx = gtk_option_menu_get_history (option_menu);
 
-	gnome_date_edit_set_time (GNOME_DATE_EDIT (data->date_dateedit), 
-				  get_file_ctime (first_image));
-}
+	gtk_widget_set_sensitive (data->date_dateedit, idx == FOLLOWING_DATE);
 
+	switch (idx) {
+	case NO_DATE:
+		break;
+	case FOLLOWING_DATE:
+		break;
+	case CURRENT_DATE:
+		gnome_date_edit_set_time (GNOME_DATE_EDIT (data->date_dateedit),
+					  time (NULL));
 
+		break;
+	case IMAGE_CREATION_DATE:
+		gnome_date_edit_set_time (GNOME_DATE_EDIT (data->date_dateedit), 
+					  get_file_ctime (first_image));
+		break;
+	case EXIF_DATE:
 #ifdef HAVE_LIBEXIF
-
-static void
-date_exif_clicked_cb (GtkWidget  *widget, 
-		       DialogData *data)
-{
-	char         *first_image = data->file_list->data;
-	ExifData     *edata;
-	unsigned int  i, j;
-	time_t        time = 0;
-	struct tm     tm = { 0, };
-
-	edata = exif_data_new_from_file (first_image);
-	if (edata == NULL) 
-		return;
-
-	for (i = 0; i < EXIF_IFD_COUNT; i++) {
-		ExifContent *content = edata->ifd[i];
-
-		if (! edata->ifd[i] || ! edata->ifd[i]->count) 
-			continue;
-
-		for (j = 0; j < content->count; j++) {
-			ExifEntry *e = content->entries[j];
-			char      *data;
-
-			if (! content->entries[j]) 
-				continue;
-
-			if ((e->tag != EXIF_TAG_DATE_TIME) &&
-			    (e->tag != EXIF_TAG_DATE_TIME_ORIGINAL) &&
-			    (e->tag != EXIF_TAG_DATE_TIME_DIGITIZED))
-				continue;
-
-			data = g_strdup (e->data);
-			data[4] = data[7] = data[10] = data[13] = data[16] = '\0';
-
-			tm.tm_year = atoi (data) - 1900;
-			tm.tm_mon  = atoi (data + 5) - 1;
-			tm.tm_mday = atoi (data + 8);
-			tm.tm_hour = atoi (data + 11);
-			tm.tm_min  = atoi (data + 14);
-			tm.tm_sec  = atoi (data + 17);
-			time = mktime (&tm);
-
-			g_free (data);
-
-			break;
-		}
+		gnome_date_edit_set_time (GNOME_DATE_EDIT (data->date_dateedit),
+					  get_exif_date (data, first_image));
+#endif
+		break;
 	}
-
-	exif_data_unref (edata);
-
-	if (time == 0)
-		return;
-
-	gnome_date_edit_set_time (GNOME_DATE_EDIT (data->date_dateedit), time);
 }
 
-#endif
+
+static GtkWidget *
+get_exif_date_option_item (DialogData *data)
+{
+	GtkWidget *menu;
+	GList     *list, *scan;
+	int        i = 0;
+	GtkWidget *retval = NULL;
+
+	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (data->date_optionmenu));
+	list = gtk_container_get_children (GTK_CONTAINER (menu));
+	for (scan = list; scan && i < EXIF_DATE; scan = scan->next)
+		i++;
+
+	retval = (GtkWidget*) scan->data;
+	g_list_free (list);
+
+	return retval;
+}
 
 
 
@@ -341,7 +400,6 @@ dlg_edit_comment (GtkWidget *widget, gpointer wdata)
 {
 	GThumbWindow      *window = wdata;
 	DialogData        *data;
-	ImageList         *ilist;
 	GtkWidget         *btn_ok;
 	GtkWidget         *btn_cancel;
 	GtkWidget         *btn_help;
@@ -349,18 +407,22 @@ dlg_edit_comment (GtkWidget *widget, gpointer wdata)
 	GList             *scan;
 	char              *first_image;
 
-	data = g_new (DialogData, 1);
+	if (window->comments_dlg != NULL) {
+		gtk_window_present (GTK_WINDOW (window->comments_dlg));
+		return;
+	}
+
+	data = g_new0 (DialogData, 1);
 
 	data->window = window;
 
-	data->gui = glade_xml_new (GTHUMB_GLADEDIR "/" COMMENT_GLADE_FILE , NULL, NULL);
+	data->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_FILE , NULL, NULL);
         if (!data->gui) {
                 g_warning ("Could not find " GLADE_FILE "\n");
                 return;
         }
 
-	ilist = IMAGE_LIST (window->file_list->ilist);
-	data->file_list = ilist_utils_get_file_list_selection (ilist);
+	data->file_list = gth_file_view_get_file_list_selection (window->file_list->view);
 	if (data->file_list == NULL) {
 		g_free (data);
 		return;
@@ -368,35 +430,33 @@ dlg_edit_comment (GtkWidget *widget, gpointer wdata)
 
 	/* Get the widgets. */
 
-	data->dialog = glade_xml_get_widget (data->gui, "comments_dialog");
-
-	data->place_entry = glade_xml_get_widget (data->gui, "place_entry");
-	data->date_checkbutton = glade_xml_get_widget (data->gui, "date_checkbutton");
-	data->date_dateedit = glade_xml_get_widget (data->gui, "date_dateedit");
-	data->date_today_button = glade_xml_get_widget (data->gui, "date_today_button");
-	data->date_ctime_button = glade_xml_get_widget (data->gui, "date_ctime_button");
-	data->date_exif_button = glade_xml_get_widget (data->gui, "date_exif_button");
+	window->comments_dlg = data->dialog = glade_xml_get_widget (data->gui, "comments_dialog");
 	data->note_text_view = glade_xml_get_widget (data->gui, "note_text");
-
+	data->place_entry = glade_xml_get_widget (data->gui, "place_entry");
+	data->date_optionmenu = glade_xml_get_widget (data->gui, "date_optionmenu");
+	data->date_dateedit = glade_xml_get_widget (data->gui, "date_dateedit");
 	data->save_changed_checkbutton = glade_xml_get_widget (data->gui, "save_changed_checkbutton");
-
         btn_ok = glade_xml_get_widget (data->gui, "ok_button");
         btn_cancel = glade_xml_get_widget (data->gui, "close_button");
 	btn_help = glade_xml_get_widget (data->gui, "help_button");
 
 	/* Set widgets data. */
 
+	gtk_option_menu_set_history (GTK_OPTION_MENU (data->date_optionmenu), NO_DATE);
+
 	data->note_text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (data->note_text_view));
 
-	if (g_list_length (data->file_list) > 1) {
+	data->several_images = g_list_length (data->file_list) > 1;
+
+	if (data->several_images) 
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->save_changed_checkbutton), TRUE);
-	} else
+	else
 		gtk_widget_set_sensitive (data->save_changed_checkbutton, FALSE);
 
 	/**/
 
 #ifndef HAVE_LIBEXIF
-	gtk_widget_hide (data->date_exif_button);
+	/* gtk_widget_hide (data->cd_exif_date_radiobutton); FIXME */
 	data->have_exif_data = FALSE;
 #else
 	{
@@ -421,7 +481,6 @@ dlg_edit_comment (GtkWidget *widget, gpointer wdata)
 		for (scan = data->file_list->next; scan; scan = scan->next) {
 			CommentData *scan_cdata;
 
-
 			scan_cdata = comments_load_comment (scan->data);
 			if (scan_cdata == NULL)
 				break;
@@ -447,38 +506,38 @@ dlg_edit_comment (GtkWidget *widget, gpointer wdata)
 		}
 	}
 
+	gtk_widget_set_sensitive (get_exif_date_option_item (data), data->have_exif_data);
 	gtk_widget_set_sensitive (data->date_dateedit, FALSE);
 
 	if (cdata != NULL) {
-		char *utf8;
+		if (cdata->comment != NULL) {
+			GtkTextIter iter;
 
-		if (cdata->comment != NULL)
-			utf8 = g_locale_to_utf8 (cdata->comment, -1,
-						 NULL, NULL, NULL);
-		else
-			utf8 = NULL;
-		if (cdata->comment != NULL)
 			gtk_text_buffer_set_text (data->note_text_buffer,
-						  utf8, 
-						  strlen (utf8));
-		g_free (utf8);
+						  cdata->comment, 
+						  strlen (cdata->comment));
+
+			gtk_text_buffer_get_iter_at_line (data->note_text_buffer,
+							  &iter,
+							  0);
+			gtk_text_buffer_place_cursor (data->note_text_buffer,
+						      &iter);
+		}
 
 		/**/
 
-		if (cdata->place != NULL)
-			_gtk_entry_set_locale_text (GTK_ENTRY (data->place_entry), cdata->place);
+		if (cdata->place != NULL) 
+			gtk_entry_set_text (GTK_ENTRY (data->place_entry), cdata->place);
 
 		/**/
 
 		if (cdata->time > 0) {
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->date_checkbutton), TRUE);
+			gtk_option_menu_set_history (GTK_OPTION_MENU (data->date_optionmenu), FOLLOWING_DATE);
 			gnome_date_edit_set_time (GNOME_DATE_EDIT (data->date_dateedit), cdata->time);
 			gtk_widget_set_sensitive (data->date_dateedit, TRUE);
-			gtk_widget_set_sensitive (data->date_today_button, TRUE);
-			gtk_widget_set_sensitive (data->date_ctime_button, TRUE);
-			gtk_widget_set_sensitive (data->date_exif_button, data->have_exif_data);
 		} else 
 			gnome_date_edit_set_time (GNOME_DATE_EDIT (data->date_dateedit), get_file_ctime (first_image));
+
 	} else 
 		gnome_date_edit_set_time (GNOME_DATE_EDIT (data->date_dateedit), get_file_ctime (first_image));
 
@@ -500,24 +559,11 @@ dlg_edit_comment (GtkWidget *widget, gpointer wdata)
 			  "clicked",
 			  G_CALLBACK (help_cb),
 			  data);
-	g_signal_connect (G_OBJECT (data->date_checkbutton),
-			  "toggled",
-			  G_CALLBACK (date_check_toggled_cb),
+
+	g_signal_connect (G_OBJECT (data->date_optionmenu), 
+			  "changed",
+			  G_CALLBACK (date_optionmenu_changed_cb),
 			  data);
-	g_signal_connect (G_OBJECT (data->date_today_button), 
-			  "clicked",
-			  G_CALLBACK (date_today_clicked_cb),
-			  data);
-	g_signal_connect (G_OBJECT (data->date_ctime_button), 
-			  "clicked",
-			  G_CALLBACK (date_ctime_clicked_cb),
-			  data);
-#ifdef HAVE_LIBEXIF
-	g_signal_connect (G_OBJECT (data->date_exif_button), 
-			  "clicked",
-			  G_CALLBACK (date_exif_clicked_cb),
-			  data);
-#endif
 
 	/* run dialog. */
 
@@ -525,6 +571,6 @@ dlg_edit_comment (GtkWidget *widget, gpointer wdata)
 
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), 
 				      GTK_WINDOW (window->app));
-	gtk_window_set_modal (GTK_WINDOW (data->dialog), TRUE);
+	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
 	gtk_widget_show (data->dialog);
 }
