@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@
 #define CONVERT_GLADE_FILE "gthumb_convert.glade"
 #define PROGRESS_GLADE_FILE "gthumb_png_exporter.glade"
 #define MAX_NAME_LEN 1024
+#define DEF_TYPE "jpeg"
 
 
 typedef struct {
@@ -69,11 +70,11 @@ typedef struct {
 	GList        *file_list;
 	GList        *current_image;
 
-	int           images;
-	int           image;
-	gboolean      remove_original;
-	gboolean      stop_convertion;
-	OverwriteMode overwrite_mode;
+	int              images;
+	int              image;
+	gboolean         remove_original;
+	gboolean         stop_convertion;
+	GthOverwriteMode overwrite_mode;
 
 	ImageLoader  *loader;
 	GdkPixbuf    *pixbuf;
@@ -96,10 +97,7 @@ destroy_cb (GtkWidget  *widget,
 	if (data->loader != NULL)
 		g_object_unref (data->loader);
 
-	if (data->file_list != NULL) {
-		g_list_foreach (data->file_list, (GFunc) file_data_unref, NULL);
-		g_list_free (data->file_list);
-	}
+	file_data_list_free (data->file_list);
 
 	g_strfreev (data->keys);
 	g_strfreev (data->values);	
@@ -152,12 +150,7 @@ load_current_image (DialogData *data)
 	g_free (folder);
 	g_free (name_no_ext);
 
-	if (strcmp (fd->path, data->new_path) == 0) {
-		load_next_image (data);
-		return;
-	}
-
-	utf8_name = g_locale_to_utf8 (file_name_from_path (data->new_path), -1, 0, 0, 0);
+	utf8_name = g_filename_to_utf8 (file_name_from_path (data->new_path), -1, 0, 0, 0);
 	message = g_strdup_printf (_("Converting image: %s"), utf8_name);
 
 	gtk_label_set_text (GTK_LABEL (data->progress_label), message);
@@ -179,7 +172,7 @@ show_rename_dialog (DialogData *data)
 	char  *message;
 	char  *utf8_name;
 
-	utf8_name = g_locale_to_utf8 (file_name_from_path (data->new_path), -1, 0, 0, 0);
+	utf8_name = g_filename_to_utf8 (file_name_from_path (data->new_path), -1, 0, 0, 0);
 
 	message = g_strdup_printf (_("An image named \"%s\" is already present. " "Please specify a different name."), utf8_name);
 	
@@ -198,7 +191,10 @@ show_rename_dialog (DialogData *data)
 static void
 save_image_and_remove_original (DialogData *data)
 {
-	GError *error = NULL;
+	GError   *error = NULL;
+
+	if (path_is_file (data->new_path))
+		unlink (data->new_path);
 
 	if (_gdk_pixbuf_savev (data->pixbuf, 
 			       data->new_path, 
@@ -206,14 +202,15 @@ save_image_and_remove_original (DialogData *data)
 			       data->keys, 
 			       data->values,
 			       &error)) {
-		GList *list;
+		GList    *list;
+		FileData *fd = data->current_image->data;
 
 		list = g_list_prepend (NULL, data->new_path);
 		all_windows_notify_files_created (list);
 		g_list_free (list);
 
-		if (data->remove_original) {
-			FileData *fd = data->current_image->data;
+		if (data->remove_original 
+		    && (strcmp (fd->path, data->new_path) != 0)) {
 			unlink (fd->path);
 
 			list = g_list_prepend (NULL, fd->path);
@@ -235,7 +232,7 @@ rename_response_cb (GtkWidget  *dialog,
 	if (response_id == GTK_RESPONSE_OK) {
 		char *new_name, *folder;
 
-		new_name = _gtk_entry_get_locale_text (GTK_ENTRY (data->conv_ren_name_entry));
+		new_name = _gtk_entry_get_filename_text (GTK_ENTRY (data->conv_ren_name_entry));
 		folder = remove_level_from_path (data->new_path);
 
 		g_free (data->new_path);
@@ -297,22 +294,22 @@ loader_done (ImageLoader *il,
 		char      *utf8_name;
 
 		switch (data->overwrite_mode) {
-		case OVERWRITE_SKIP:
+		case GTH_OVERWRITE_SKIP:
 			save_image = FALSE;
 			break;
 
-		case OVERWRITE_OVERWRITE:
+		case GTH_OVERWRITE_OVERWRITE:
 			save_image = TRUE;
 			break;
 
-		case OVERWRITE_ASK:
-			utf8_name = g_locale_to_utf8 (file_name_from_path (data->new_path), -1, 0, 0, 0);
+		case GTH_OVERWRITE_ASK:
+			utf8_name = g_filename_to_utf8 (file_name_from_path (data->new_path), -1, 0, 0, 0);
 			message = g_strdup_printf (_("An image named \"%s\" is already present. " "Do you want to overwrite it?"), utf8_name);
 
 			d = _gtk_yesno_dialog_new (GTK_WINDOW (data->dialog),
 						   GTK_DIALOG_MODAL,
 						   message,
-						   GTK_STOCK_CANCEL,
+						   _("Skip"),
 						   _("_Overwrite"));
 
 			g_signal_connect (G_OBJECT (d), "response",
@@ -325,7 +322,7 @@ loader_done (ImageLoader *il,
 			gtk_widget_show (d);
 			return;
 			
-		case OVERWRITE_RENAME:
+		case GTH_OVERWRITE_RENAME:
 			show_rename_dialog (data);
 			return;
 		}
@@ -435,7 +432,6 @@ dlg_convert (GThumbWindow *window)
 
 	data = g_new0 (DialogData, 1);
 
-	g_list_foreach (list, (GFunc) file_data_ref, NULL);
 	data->file_list = list;
 	data->current_image = list;
 
@@ -491,7 +487,7 @@ dlg_convert (GThumbWindow *window)
 
 	/* Set default values. */
 
-	image_type = eel_gconf_get_string (PREF_CONVERT_IMAGE_TYPE);
+	image_type = eel_gconf_get_string (PREF_CONVERT_IMAGE_TYPE, DEF_TYPE);
 
 	if (image_type == NULL)
 		button = data->conv_jpeg_radiobutton;
@@ -514,7 +510,7 @@ dlg_convert (GThumbWindow *window)
 				     pref_get_convert_overwrite_mode ());
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->conv_remove_orig_checkbutton), 
-				      eel_gconf_get_boolean (PREF_CONVERT_REMOVE_ORIGINAL));
+				      eel_gconf_get_boolean (PREF_CONVERT_REMOVE_ORIGINAL, FALSE));
 
 	/* Set the signals handlers. */
 	
